//
//  EmbededViewController.swift
//  KakaoPartnerSDKSample
//
//  Created by kakao on 2/3/25.
//  Copyright © 2025 apiteam. All rights reserved.
//


import UIKit
import KakaoSDKCommon
import KakaoSDKAuth
import KakaoPartnerSDKFriendDelegate

class EmbededViewController: UIViewController {
    private var pickerView: PickerView?
    
    private let naviView: UIView = {
        let view = UIView()
        view.translatesAutoresizingMaskIntoConstraints = false
        view.backgroundColor = .systemBackground
        return view
    }()
    
    private let okBtn: UIButton = {
        let button = UIButton()
        button.setTitle("Ok", for: .normal)
        button.setTitleColor(.black, for: .normal)
        button.addTarget(self, action: #selector(getSelectedUsers), for: .touchUpInside)
        
        return button
    }()
    
    override func loadView() {
        super.loadView()
        view.backgroundColor = .systemBackground
    }
    
    override func viewDidLoad() {
        setNaviViewLayout()
        getPickerView()
    }
    
    private func setNaviViewLayout() {
        view.addSubview(naviView)
        
        naviView.topAnchor.constraint(equalTo: view.safeAreaLayoutGuide.topAnchor).isActive = true
        naviView.leadingAnchor.constraint(equalTo: view.safeAreaLayoutGuide.leadingAnchor).isActive = true
        naviView.widthAnchor.constraint(equalTo: view.widthAnchor).isActive = true
        naviView.heightAnchor.constraint(equalToConstant: 50).isActive = true
        
        setOkButtonLayout()
    }
    
    private func setOkButtonLayout() {
        naviView.addSubview(okBtn)
        
        okBtn.translatesAutoresizingMaskIntoConstraints = false
        okBtn.topAnchor.constraint(equalTo: naviView.topAnchor).isActive = true
        okBtn.trailingAnchor.constraint(equalTo: naviView.trailingAnchor, constant: -10).isActive = true
        okBtn.bottomAnchor.constraint(equalTo: naviView.bottomAnchor).isActive = true
    }
    
    private func getPickerView() {
        guard let appKey = try? KakaoSDK.shared.appKey(),
              let bundleId = Bundle.main.bundleIdentifier,
              let accessToken = TokenManager.manager.getToken()?.accessToken else {
            dismiss(animated: true)
            
            return
        }
                
        let params = PickerFriendRequestParams(showFavorite: false)
        let targetInfo = TargetInfo(appKey: appKey, bundleId: bundleId, accessToken: accessToken)
        
        PickerApi.shared.singleFriendPickerView(
            params: params,
            targetInfo: targetInfo
        ) { [weak self] resultView, error in
            if let error {
                print("##### error occur: \(error)")
                
                DispatchQueue.main.async {
                    self?.dismiss(animated: true)
                }
            }
            
            if let resultView {
                self?.pickerView = resultView
                self?.setLayout()
                self?.setDelegate()
            }
        }
    }
    
    private func setLayout() {
        guard let pickerView else { return }
        
        view.addSubview(pickerView)
        pickerView.translatesAutoresizingMaskIntoConstraints = false
        pickerView.topAnchor.constraint(equalTo: naviView.bottomAnchor).isActive = true
        pickerView.leadingAnchor.constraint(equalTo: view.safeAreaLayoutGuide.leadingAnchor).isActive = true
        pickerView.trailingAnchor.constraint(equalTo: view.safeAreaLayoutGuide.trailingAnchor).isActive = true
        pickerView.bottomAnchor.constraint(equalTo: view.safeAreaLayoutGuide.bottomAnchor).isActive = true
        
        pickerView.setLayout()
        pickerView.updateSubviewsConstraint()
    }
    
    private func setDelegate() {
        guard let pickerView else { return }
        
        pickerView.setCustomFavoriteCallback(self)
        pickerView.setSelectedUserCallback(self)
    }
    
    override func viewWillTransition(to size: CGSize, with coordinator: any UIViewControllerTransitionCoordinator) {
        coordinator.animate { [weak self] _ in
            self?.pickerView?.updateSubviewsConstraint()
        }
        super.viewWillTransition(to: size, with: coordinator)
    }
}

extension EmbededViewController {
    func showAlert(value: Any) {
        DispatchQueue.main.async { [weak self] in
            let alert = UIAlertController(title: "Info", message: String(describing: value), preferredStyle: .alert)
            let okAction = UIAlertAction(title: "Ok", style: .default) { _ in
                self?.dismiss(animated: true)
            }
            alert.addAction(okAction)
            
            self?.present(alert, animated: true)
        }
    }
    
    @objc func getSelectedUsers() {
        pickerView?.result(completion: { [weak self] users in
            self?.showAlert(value: users)
        })
    }
}

// MARK: - SelectedUserCallback
extension EmbededViewController: SelectedUserCallback {
    func select(user: SelectedUsers) {
        showAlert(value: user)
    }
}

// MARK: - FavoriteUserDelegate
extension EmbededViewController: CustomFavoriteCallback {
    func selectUser(uuid: String, appId: Int64?) {
        print("##### favorite user: \(uuid)")
    }
    
    func deselectUser(uuid: String, appId: Int64?) {
        print("##### not favorite user: \(uuid)")
    }
}
